@@ -41,70 +41,77 @@ class Builder():
 
     # Map PyQt modules to the corresponding qmake QT or CONFIG values.  A
     # module that doesn't need any of these values can be omitted
-    class _QtMetaData:
-        def __init__(self, qt=None, config=None):
+    class _Qt5MetaData:
+        def __init__(self, qt=None, config=None, needs_suffix=False):
             self.qt = qt
             self.config = config
+            self.needs_suffix = needs_suffix
+
+    class _Qt4MetaData(_Qt5MetaData):
+        def __init__(self, qt=None, config=None, needs_suffix=True):
+            super().__init__(qt=qt, config=config, needs_suffix=needs_suffix)
 
     _pyqt5_module_map = {
-        'QAxContainer':         _QtMetaData(qt=['axcontainer']),
-        'QtBluetooth':          _QtMetaData(qt=['bluetooth']),
-        'QtCore':               _QtMetaData(qt=['-gui']),
-        'QtDBus':               _QtMetaData(qt=['dbus', '-gui']),
-        'QtDesigner':           _QtMetaData(qt=['designer']),
-        'QtHelp':               _QtMetaData(qt=['help']),
-        'QtMacExtras':          _QtMetaData(qt=['macextras']),
-        'QtMultimedia':         _QtMetaData(qt=['multimedia']),
-        'QtMultimediaWidgets':  _QtMetaData(
+        'QAxContainer':         _Qt5MetaData(qt=['axcontainer']),
+        'QtBluetooth':          _Qt5MetaData(qt=['bluetooth']),
+        'QtCore':               _Qt5MetaData(qt=['-gui']),
+        'QtDBus':               _Qt5MetaData(qt=['dbus', '-gui']),
+        'QtDesigner':           _Qt5MetaData(qt=['designer']),
+        'QtHelp':               _Qt5MetaData(qt=['help']),
+        'QtMacExtras':          _Qt5MetaData(qt=['macextras']),
+        'QtMultimedia':         _Qt5MetaData(qt=['multimedia']),
+        'QtMultimediaWidgets':  _Qt5MetaData(
                                         qt=['multimediawidgets',
                                                 'multimedia']),
-        'QtNetwork':            _QtMetaData(qt=['network', '-gui']),
-        'QtOpenGL':             _QtMetaData(qt=['opengl']),
-        'QtPositioning':        _QtMetaData(qt=['positioning']),
-        'QtPrintSupport':       _QtMetaData(qt=['printsupport']),
-        'QtQml':                _QtMetaData(qt=['qml']),
-        'QtQuick':              _QtMetaData(qt=['quick']),
-        'QtSensors':            _QtMetaData(qt=['sensors']),
-        'QtSerialPort':         _QtMetaData(qt=['serialport']),
-        'QtSql':                _QtMetaData(qt=['sql', 'widgets']),
-        'QtSvg':                _QtMetaData(qt=['svg']),
-        'QtTest':               _QtMetaData(qt=['testlib', 'widgets']),
-        'QtWebKit':             _QtMetaData(qt=['webkit', 'network']),
-        'QtWebKitWidgets':      _QtMetaData(qt=['webkitwidgets']),
-        'QtWidgets':            _QtMetaData(qt=['widgets']),
-        'QtWinExtras':          _QtMetaData(qt=['winextras', 'widgets']),
-        'QtX11Extras':          _QtMetaData(qt=['x11extras']),
-        'QtXmlPatterns':        _QtMetaData(
+        'QtNetwork':            _Qt5MetaData(qt=['network', '-gui']),
+        'QtOpenGL':             _Qt5MetaData(qt=['opengl']),
+        'QtPositioning':        _Qt5MetaData(qt=['positioning']),
+        'QtPrintSupport':       _Qt5MetaData(qt=['printsupport']),
+        'QtQml':                _Qt5MetaData(qt=['qml']),
+        'QtQuick':              _Qt5MetaData(qt=['quick']),
+        'QtSensors':            _Qt5MetaData(qt=['sensors']),
+        'QtSerialPort':         _Qt5MetaData(qt=['serialport']),
+        'QtSql':                _Qt5MetaData(qt=['sql', 'widgets']),
+        'QtSvg':                _Qt5MetaData(qt=['svg']),
+        'QtTest':               _Qt5MetaData(qt=['testlib', 'widgets']),
+        'QtWebKit':             _Qt5MetaData(qt=['webkit', 'network']),
+        'QtWebKitWidgets':      _Qt5MetaData(qt=['webkitwidgets']),
+        'QtWidgets':            _Qt5MetaData(qt=['widgets']),
+        'QtWinExtras':          _Qt5MetaData(qt=['winextras', 'widgets']),
+        'QtX11Extras':          _Qt5MetaData(qt=['x11extras']),
+        'QtXmlPatterns':        _Qt5MetaData(
                                         qt=['xmlpatterns', '-gui', 'network']),
 
-        'QtChart':              _QtMetaData(config=['qtcommercialchart']),
-        'QtDataVisualization':  _QtMetaData(qt=['datavisualization']),
-        'Qsci':                 _QtMetaData(config=['qscintilla2']),
+        'QtChart':              _Qt5MetaData(config=['qtcommercialchart']),
+        'QtDataVisualization':  _Qt5MetaData(qt=['datavisualization']),
+        'Qsci':                 _Qt5MetaData(config=['qscintilla2']),
     }
 
     _pyqt4_module_map = {
-        'QAxContainer':         _QtMetaData(config=['qaxcontainer']),
-        'QtCore':               _QtMetaData(qt=['-gui']),
-        'QtDBus':               _QtMetaData(qt=['dbus', '-gui']),
-        'QtDeclarative':        _QtMetaData(qt=['declarative', 'network']),
-        'QtDesigner':           _QtMetaData(config=['designer']),
-        'QtHelp':               _QtMetaData(config=['help']),
-        'QtMultimedia':         _QtMetaData(qt=['multimedia']),
-        'QtNetwork':            _QtMetaData(qt=['network', '-gui']),
-        'QtOpenGL':             _QtMetaData(qt=['opengl']),
-        'QtScript':             _QtMetaData(qt=['script', '-gui']),
-        'QtScriptTools':        _QtMetaData(qt=['scripttools', 'script']),
-        'QtSql':                _QtMetaData(qt=['sql']),
-        'QtSvg':                _QtMetaData(qt=['svg']),
-        'QtTest':               _QtMetaData(qt=['testlib']),
-        'QtWebKit':             _QtMetaData(qt=['webkit', 'network']),
-        'QtXml':                _QtMetaData(qt=['xml', '-gui']),
-        'QtXmlPatterns':        _QtMetaData(
+        'QAxContainer':         _Qt4MetaData(config=['qaxcontainer']),
+        'QtCore':               _Qt4MetaData(qt=['-gui']),
+        'QtDBus':               _Qt4MetaData(qt=['dbus', '-gui']),
+        'QtDeclarative':        _Qt4MetaData(qt=['declarative', 'network']),
+        'QtDesigner':           _Qt4MetaData(config=['designer']),
+        'QtHelp':               _Qt4MetaData(config=['help']),
+        'QtMultimedia':         _Qt4MetaData(qt=['multimedia']),
+        'QtNetwork':            _Qt4MetaData(qt=['network', '-gui']),
+        'QtOpenGL':             _Qt4MetaData(qt=['opengl']),
+        'QtScript':             _Qt4MetaData(qt=['script', '-gui']),
+        'QtScriptTools':        _Qt4MetaData(qt=['scripttools', 'script']),
+        'QtSql':                _Qt4MetaData(qt=['sql']),
+        'QtSvg':                _Qt4MetaData(qt=['svg']),
+        'QtTest':               _Qt4MetaData(qt=['testlib']),
+        'QtWebKit':             _Qt4MetaData(qt=['webkit', 'network']),
+        'QtXml':                _Qt4MetaData(qt=['xml', '-gui']),
+        'QtXmlPatterns':        _Qt4MetaData(
                                         qt=['xmlpatterns', '-gui', 'network']),
-        'phonon':               _QtMetaData(qt=['phonon']),
+        'phonon':               _Qt4MetaData(qt=['phonon']),
 
-        'QtChart':              _QtMetaData(config=['qtcommercialchart']),
-        'Qsci':                 _QtMetaData(config=['qscintilla2']),
+        'QtChart':              _Qt4MetaData(config=['qtcommercialchart'],
+                                        needs_suffix=False),
+        'Qsci':                 _Qt4MetaData(config=['qscintilla2'],
+                                        needs_suffix=False),
     }
 
     def __init__(self, project, verbose=False):
@@ -211,12 +218,10 @@ class Builder():
         qmake_qt = []
         qmake_config = []
 
-        module_map = self._pyqt5_module_map if project.application_is_pyqt5 else self._pyqt4_module_map
-
         for pyqt_m in project.pyqt_modules:
             needs_gui = True
 
-            metadata = module_map.get(pyqt_m, self._QtMetaData())
+            metadata = self._get_metadata(pyqt_m)
 
             if metadata.qt is not None:
                 for qt in metadata.qt:
@@ -234,6 +239,7 @@ class Builder():
                 no_gui = False
 
         if no_gui:
+            f.write('CONFIG += console\n')
             f.write('QT -= gui\n')
 
         if len(qmake_qt) != 0:
@@ -247,31 +253,44 @@ class Builder():
 
         for extension_module in project.extension_modules:
             if extension_module.name != '':
-                extensions[extension_module.name] = extension_module.path
+                extensions[extension_module.name] = (
+                        QDir.fromNativeSeparators(
+                                project.absolute_path(extension_module.path)),
+                        extension_module.name)
 
         if len(project.pyqt_modules) > 0:
-            sitepackages = project.absolute_path(
-                    project.python_target_stdlib_dir) + '/site-packages'
+            sitepackages = QDir.fromNativeSeparators(
+                    project.absolute_path(
+                            project.python_target_stdlib_dir)) + '/site-packages'
             pyqt_version = 'PyQt5' if project.application_is_pyqt5 else 'PyQt4'
 
             for pyqt in project.pyqt_modules:
                 if pyqt != 'uic':
-                    extensions[pyqt_version + '.' + pyqt] = sitepackages + '/' + pyqt_version
+                    lib_name = pyqt
+                    if self._get_metadata(pyqt).needs_suffix:
+                        # Qt4's qmake thinks -lQtCore etc. always refer to the
+                        # Qt libraries so PyQt4 creates static libraries with a
+                        # suffix.
+                        lib_name += '_s'
+
+                    extensions[pyqt_version + '.' + pyqt] = (
+                            sitepackages + '/' + pyqt_version,
+                            lib_name)
 
             # Add the implicit sip module.
-            extensions['sip'] = sitepackages
+            extensions['sip'] = (sitepackages, 'sip')
 
         if len(extensions) > 0:
             f.write('\n')
 
             # Get the list of unique module directories.
             mod_dirs = []
-            for mod_dir in extensions.values():
+            for mod_dir, _ in extensions.values():
                 if mod_dir not in mod_dirs:
                     mod_dirs.append(mod_dir)
 
-            mod_dir_flags = ['-L' + md for md in mod_dirs]
-            mod_flags = ['-l' + m.split('.')[-1] for m in extensions.keys()]
+            mod_dir_flags = ['-L' + self._quote(md) for md in mod_dirs]
+            mod_flags = ['-l' + l for _, l in extensions.values()]
 
             f.write(
                     'LIBS += {0} {1}\n'.format(' '.join(mod_dir_flags),
@@ -281,19 +300,21 @@ class Builder():
         f.write('\n')
 
         if project.python_target_include_dir != '':
-            f.write(
-                    'INCLUDEPATH += {0}\n'.format(
-                            project.python_target_include_dir))
+            inc_dir = QDir.fromNativeSeparators(
+                    project.absolute_path(project.python_target_include_dir))
+            f.write('INCLUDEPATH += {0}\n'.format(self._quote(inc_dir)))
 
         if project.python_target_library != '':
             lib_dir = os.path.dirname(project.python_target_library)
+            lib_dir = QDir.fromNativeSeparators(project.absolute_path(lib_dir))
+
             lib, _ = os.path.splitext(
                     os.path.basename(project.python_target_library))
 
             if lib.startswith('lib'):
                 lib = lib[3:]
 
-            f.write('LIBS += -L{0} -l{1}\n'.format(lib_dir, lib))
+            f.write('LIBS += -L{0} -l{1}\n'.format(self._quote(lib_dir), lib))
 
         # Add the platform specific stuff.
         platforms_f = QFile(self._lib_filename('platforms.pro'))
@@ -335,21 +356,19 @@ class Builder():
 
         f.write('HEADERS = frozen_bootstrap.h frozen_main.h pyqtdeploy_version.h\n')
 
-        bootstrap_f = self._create_file(build_dir, '__bootstrap__.py')
-        bootstrap_f.write('''import sys
-import pyqtdeploy
+        major, _ = project.python_target_version
+        if major is None:
+            raise UserException("Unable to determine target Python version")
 
-sys.path = [{0}]
-sys.path_hooks = [pyqtdeploy.qrcimporter]
-'''.format(', '.join(["':/{0}'".format(resource) for resource in resources])))
-        bootstrap_f.close()
-
-        self._freeze(os.path.join(build_dir, 'frozen_bootstrap.h'),
-                os.path.join(build_dir, '__bootstrap__.py'), freeze)
+        bootstrap = self._copy_lib_file(
+                'bootstrap_py3.py' if major == 3 else 'bootstrap_py2.py')
+        self._freeze(os.path.join(build_dir, 'frozen_bootstrap.h'), bootstrap,
+                freeze, name='pyqtdeploy_bootstrap')
+        os.remove(bootstrap)
 
         self._freeze(os.path.join(build_dir, 'frozen_main.h'),
                 project.absolute_path(project.application_script), freeze,
-                main=True)
+                name='pyqtdeploy_main')
 
         version_f = self._create_file(build_dir, 'pyqtdeploy_version.h')
         version_f.write(
@@ -384,6 +403,27 @@ sys.path_hooks = [pyqtdeploy.qrcimporter]
 
         return resources
 
+    def _get_metadata(self, name):
+        """ Get the metadata for a module. """
+
+        if self._project.application_is_pyqt5:
+            module_map = self._pyqt5_module_map
+            default_metadata = self._Qt5MetaData()
+        else:
+            module_map = self._pyqt4_module_map
+            default_metadata = self._Qt4MetaData()
+
+        return module_map.get(name, default_metadata)
+
+    @staticmethod
+    def _quote(name):
+        """ Return the quoted version of a name if quoting is required. """
+
+        if ' ' in name:
+            name = '"' + name + '"'
+
+        return name
+
     def _write_resource(self, resources_dir, resource, package, src, freeze):
         """ Create a .qrc file for a resource and the corresponding contents.
         """
@@ -556,13 +596,14 @@ int main(int argc, char **argv)
     return pyqtdeploy_main(argc, argv, %s"%s", %s);
 ''' % (name_type, name_prefix, app_name, inittab))
 
-    def _freeze(self, output, py_filename, freeze, main=False, as_data=False):
+    def _freeze(self, output, py_filename, freeze, name=None, as_data=False):
         """ Freeze a Python source file to a C header file or a data file. """
 
         args = [self._project.python_host_interpreter, freeze]
         
-        if main:
-            args.append('--main')
+        if name is not None:
+            args.append('--name')
+            args.append(name)
 
         args.append('--as-data' if as_data else '--as-c')
         args.append(output)
