@@ -87,7 +87,11 @@ class _StdlibPackageEditor(QrcPackageEditor):
     _title = "Standard Library"
 
     # The required Python v3 modules.
-    _py3_required = ('importlib', 'types.py', 'warnings.py')
+    _py3_required = ('_weakrefset.py', 'abc.py', 'codecs.py',
+        'encodings/__init__.py', 'encodings/aliases.py', 'encodings/ascii.py',
+        'encodings/cp437.py', 'encodings/latin_1.py', 'encodings/mbcs.py',
+        'encodings/utf_8.py', 'importlib/__init__.py', 'io.py', 'types.py',
+        'warnings.py')
 
     # The required Python v2 modules.
     _py2_required = ('atexit.py', )
@@ -98,11 +102,34 @@ class _StdlibPackageEditor(QrcPackageEditor):
         super().__init__(self._title)
 
         self._project = None
-        self._required = self._py3_required
+        self._py_version = None
 
     def get_root_dir(self):
         """ Get the name of the Python standard library directory. """
 
+        major, minor = self._project.python_target_version
+
+        if major is None:
+            QMessageBox.warning(self, self._title,
+                    "The standard library cannot be scanned because the "
+                    "Python version cannot be obtained from the Python "
+                    "library name in the Python Configuration tab.")
+            return ''
+
+        if major == 3 and minor < 3:
+            QMessageBox.warning(self, self._title,
+                    "When targetting Python v3, Python v3.3 or later is "
+                    "required.")
+            return ''
+
+        if major == 2 and minor < 6:
+            QMessageBox.warning(self, self._title,
+                    "When targetting Python v2, Python v2.6 or later is "
+                    "required.")
+            return ''
+
+        self._py_version = major
+
         stdlib_dir = self._project.python_target_stdlib_dir
 
         if stdlib_dir == '':
@@ -112,27 +139,22 @@ class _StdlibPackageEditor(QrcPackageEditor):
                     "Configuration tab.")
             return ''
 
-        self._required = self._py3_required
-
         return stdlib_dir
 
     def filter(self, name):
         """ Reimplemented to filter out site-packages. """
 
-        if name in ('importlib._bootstrap.py', 'site-packages'):
+        if name == 'site-packages':
             return True
 
         return super().filter(name)
 
     def required(self, name):
-        """ Reimplemented to filter out site-packages. """
+        """ See if a name is required. """
 
-        # Remember if we seem to be scanning Python v2.  (atexit is a builtin
-        # in Python v3.)
-        if name == 'atexit.py':
-            self._required = self._py2_required
+        required_modules = self._py3_required if self._py_version == 3 else self._py2_required
 
-        if name in self._required:
+        if name in required_modules:
             return True
 
         return super().filter(name)
